@@ -51,6 +51,8 @@
 #define TD_DIST_VERSION PACKAGE_VERSION
 #endif
 
+#define EXIT_UNSUPPORTED 2
+
 extern int thd_dbus_server_init(void (*exit_handler)(int));
 
 // Lock file
@@ -216,7 +218,7 @@ int main(int argc, char *argv[]) {
 			{ "config-file", 0, 0, G_OPTION_ARG_STRING, &conf_file, N_(
 					"configuration file"), NULL },
 			{ "ignore-default-control", 0, 0, G_OPTION_ARG_NONE, &ignore_default_control, N_(
-							"Ignore default CPU temperature control."
+							"Ignore default CPU temperature control. "
 							"Strictly follow thermal-conf.xml"), NULL },
 			{ "workaround-enabled", 0, 0, G_OPTION_ARG_NONE,
 						&workaround_enabled, N_(
@@ -337,7 +339,7 @@ int main(int argc, char *argv[]) {
 	}
 
 	if (adaptive) {
-		ret = thd_engine_create_adaptive_engine((bool) ignore_cpuid_check);
+		ret = thd_engine_create_adaptive_engine((bool) ignore_cpuid_check, (bool) test_mode);
 		if (ret != THD_SUCCESS) {
 			thd_log_info("--adaptive option failed on this platform\n");
 			thd_log_info("Ignoring --adaptive option\n");
@@ -351,7 +353,11 @@ int main(int argc, char *argv[]) {
 	if (ret != THD_SUCCESS) {
 		clean_up_lockfile();
 		closelog();
-		exit(EXIT_FAILURE);
+
+		if (ret == THD_ERROR)
+			exit(EXIT_UNSUPPORTED);
+		else
+			exit(EXIT_FAILURE);
 	}
 
 	// Start service requests on the D-Bus
