@@ -349,8 +349,8 @@ int cthd_cdev::thd_cdev_set_state(int set_point, int target_temp,
 	} else if (pid_param && pid_param->valid) {
 		// Handle PID param unique to a trip
 		pid.set_target_temp(target_temp);
-		ret = pid.pid_output(temperature);
-		ret += get_curr_state(true);
+		ret = pid.pid_output(temperature, get_curr_state(true) - get_min_state());
+		ret += get_min_state();
 		if (get_min_state() < get_max_state()) {
 			if (ret > get_max_state())
 				ret = get_max_state();
