@@ -31,6 +31,7 @@
 #include "thd_sys_fs.h"
 #include "thd_preference.h"
 #include "thd_pid.h"
+#include "thd_adaptive_types.h"
 
 typedef struct {
 	int zone;
@@ -66,6 +67,8 @@ protected:
 	int last_state;
 	std::vector<zone_trip_limits_t> zone_trip_limits;
 	std::string write_prefix;
+	int inc_val;
+	int dec_val;
 
 private:
 	unsigned int int_2_pow(int pow) {
@@ -77,16 +80,17 @@ private:
 	}
 	int thd_cdev_exponential_controller(int set_point, int target_temp,
 			int temperature, int state, int arg);
-
+	int thd_clamp_state_min(int _state);
+	int thd_clamp_state_max(int _state);
 public:
 	static const int default_debounce_interval = 2; // In seconds
 	cthd_cdev(unsigned int _index, std::string control_path) :
 			index(_index), cdev_sysfs(control_path.c_str()), trip_point(0), max_state(
-					0), min_state(0), curr_state(0), curr_pow(
-					0), base_pow_state(0), inc_dec_val(1), auto_down_adjust(
-					false), read_back(true), debounce_interval(
-					default_debounce_interval), last_action_time(0), trend_increase(
-					false), pid_enable(false), pid_ctrl(), last_state(0), write_prefix("") {
+					0), min_state(0), curr_state(0), curr_pow(0), base_pow_state(
+					0), inc_dec_val(1), auto_down_adjust(false), read_back(
+					true), debounce_interval(default_debounce_interval), last_action_time(
+					0), trend_increase(false), pid_enable(false), pid_ctrl(), last_state(
+					0), write_prefix(""), inc_val(0), dec_val(0) {
 	}
 
 	virtual ~cthd_cdev() {
@@ -139,6 +143,10 @@ public:
 		return curr_state;
 	}
 
+	virtual int get_curr_state(bool read_again) {
+		return curr_state;
+	}
+
 	virtual int get_min_state() {
 		return min_state;
 	}
@@ -158,12 +166,22 @@ public:
 	virtual void set_inc_dec_value(int value) {
 		inc_dec_val = value;
 	}
+
+	virtual void set_inc_value(int value) {
+		inc_val = value;
+	}
+
+	virtual void set_dec_value(int value) {
+		dec_val = value;
+	}
+
 	virtual void set_down_adjust_control(bool value) {
 		auto_down_adjust = value;
 	}
 	virtual int map_target_state(int target_valid, int target_state) {
 		return target_state;
 	}
+	virtual void set_adaptive_target(struct adaptive_target) {};
 	void set_debounce_interval(int interval) {
 		debounce_interval = interval;
 	}
@@ -238,9 +256,15 @@ public:
 	}
 
 	void cdev_dump() {
-		thd_log_info("%d: %s, C:%d MN: %d MX:%d ST:%d pt:%s rd_bk %d \n", index,
+		if (inc_val || dec_val){
+			thd_log_info("%d: %s, C:%d MN: %d MX:%d Inc ST:%d Dec ST:%d pt:%s rd_bk %d \n", index,
+				type_str.c_str(), curr_state, min_state, max_state, inc_val, dec_val,
+				get_base_path().c_str(), read_back);
+		} else {
+			thd_log_info("%d: %s, C:%d MN: %d MX:%d ST:%d pt:%s rd_bk %d \n", index,
 				type_str.c_str(), curr_state, min_state, max_state, inc_dec_val,
 				get_base_path().c_str(), read_back);
+		}
 	}
 };
 
