@@ -3,8 +3,8 @@
  * Copyright (c) 2016, Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
- * under the terms and conditions of the GNU General Public License,
- * version 2, as published by the Free Software Foundation.
+ * under the terms and conditions of the GNU General Public License
+ * version 2 or later as published by the Free Software Foundation.
  *
  * This program is distributed in the hope it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -173,7 +173,6 @@ int cthd_cdev_modem::get_modem_property(DBusConnection* conn,
 		DBusMessageIter key, var;
 
 		char *property_name;
-		dbus_bool_t enabled;
 
 		dbus_message_iter_recurse(&dict, &key);
 
@@ -341,7 +340,7 @@ void cthd_cdev_modem::update_throttling_state() {
 	conn = dbus_bus_get(DBUS_BUS_SYSTEM, &error);
 
 	if (dbus_error_is_set(&error)) {
-		thd_log_error("Error :%s", error.message);
+		thd_log_error("Error: %s", error.message);
 		dbus_error_free(&error);
 		return;
 	}
@@ -399,7 +398,7 @@ void cthd_cdev_modem::set_curr_state(int state, int arg) {
 	conn = dbus_bus_get(DBUS_BUS_SYSTEM, &error);
 
 	if (dbus_error_is_set(&error)) {
-		thd_log_error("Erro : %s", error.message);
+		thd_log_error("Error : %s", error.message);
 		dbus_error_free(&error);
 		return;
 	}
