@@ -48,12 +48,15 @@ static void *cthd_engine_thread(void *arg);
 
 cthd_engine::cthd_engine(std::string _uuid) :
 		current_cdev_index(0), current_zone_index(0), current_sensor_index(0), parse_thermal_zone_success(
-				false), parse_thermal_cdev_success(false), uuid(_uuid), parser_disabled(false), poll_timeout_msec(
-				-1), wakeup_fd(-1), uevent_fd(-1), control_mode(COMPLEMENTRY), write_pipe_fd(
+				false), parse_thermal_cdev_success(false), uuid(_uuid), parser_disabled(
+				false), adaptive_mode(false), poll_timeout_msec(-1), wakeup_fd(
+				-1), uevent_fd(-1), control_mode(COMPLEMENTRY), write_pipe_fd(
 				0), preference(0), status(true), thz_last_uevent_time(0), thz_last_temp_ind_time(
-				0), thz_last_update_event_time(0), terminate(false), genuine_intel(0), has_invariant_tsc(0), has_aperf(
-				0), proc_list_matched(false), poll_interval_sec(0), poll_sensor_mask(
-				0), fast_poll_sensor_mask(0), saved_poll_interval(0), poll_fd_cnt(0), rt_kernel(false), parser_init_done(false) {
+				0), thz_last_update_event_time(0), terminate(false), genuine_intel(
+				0), has_invariant_tsc(0), has_aperf(0), proc_list_matched(
+				false), poll_interval_sec(0), poll_sensor_mask(0), fast_poll_sensor_mask(
+				0), saved_poll_interval(0), poll_fd_cnt(0), rt_kernel(false), parser_init_done(
+				false) {
 	thd_engine = pthread_t();
 	thd_attr = pthread_attr_t();
 
@@ -171,9 +174,10 @@ bool cthd_engine::set_preference(const int pref) {
 	return true;
 }
 
-int cthd_engine::thd_engine_start(bool ignore_cpuid_check) {
+int cthd_engine::thd_engine_init(bool ignore_cpuid_check, bool adaptive) {
 	int ret;
-	int wake_fds[2];
+
+	adaptive_mode = adaptive;
 
 	if (ignore_cpuid_check) {
 		thd_log_debug("Ignore CPU ID check for MSRs \n");
@@ -186,11 +190,39 @@ int cthd_engine::thd_engine_start(bool ignore_cpuid_check) {
 				thd_log_msg("Unsupported cpu model, using thermal-conf.xml only \n");
 			} else {
 				thd_log_msg("Unsupported cpu model, use thermal-conf.xml file or run with --ignore-cpuid-check \n");
-				return THD_FATAL_ERROR;
+				return THD_ERROR;
 			}
 		}
 	}
 
+	ret = read_thermal_sensors();
+	if (ret != THD_SUCCESS) {
+		thd_log_error("Thermal sysfs Error in reading sensors\n");
+		// This is a fatal error and daemon will exit
+		return THD_FATAL_ERROR;
+	}
+
+	ret = read_cooling_devices();
+	if (ret != THD_SUCCESS) {
+		thd_log_error("Thermal sysfs Error in reading cooling devs\n");
+		// This is a fatal error and daemon will exit
+		return THD_FATAL_ERROR;
+	}
+
+	ret = read_thermal_zones();
+	if (ret != THD_SUCCESS) {
+		thd_log_error("No thermal sensors found\n");
+		// This is a fatal error and daemon will exit
+		return THD_FATAL_ERROR;
+	}
+
+	return THD_SUCCESS;
+}
+
+int cthd_engine::thd_engine_start() {
+	int ret;
+	int wake_fds[2];
+
 	check_for_rt_kernel();
 
 	// Pipe is used for communication between two processes
@@ -223,27 +255,6 @@ int cthd_engine::thd_engine_start(bool ignore_cpuid_check) {
 		poll_timeout_msec = poll_interval_sec * 1000;
 	}
 
-	ret = read_thermal_sensors();
-	if (ret != THD_SUCCESS) {
-		thd_log_error("Thermal sysfs Error in reading sensors\n");
-		// This is a fatal error and daemon will exit
-		return THD_FATAL_ERROR;
-	}
-
-	ret = read_cooling_devices();
-	if (ret != THD_SUCCESS) {
-		thd_log_error("Thermal sysfs Error in reading cooling devs\n");
-		// This is a fatal error and daemon will exit
-		return THD_FATAL_ERROR;
-	}
-
-	ret = read_thermal_zones();
-	if (ret != THD_SUCCESS) {
-		thd_log_error("No thermal sensors found\n");
-		// This is a fatal error and daemon will exit
-		return THD_FATAL_ERROR;
-	}
-
 	if (parser.platform_matched()) {
 		parser.set_default_preference();
 		int poll_secs = parser.get_polling_interval();
@@ -694,11 +705,21 @@ static supported_ids_t id_table[] = {
 		{ 6, 0x7e }, // Icelake
 		{ 6, 0x8c }, // Tigerlake_L
 		{ 6, 0x8d }, // Tigerlake
+		{ 6, 0xa5 }, // Cometlake
+		{ 6, 0xa6 }, // Cometlake_L
+		{ 6, 0xa7 }, // Rocketlake
+		{ 6, 0x9c }, // Jasper Lake
+		{ 6, 0x97 }, // Alderlake
+		{ 6, 0x9a }, // Alderlake
 		{ 0, 0 } // Last Invalid entry
 };
 
 std::vector<std::string> blocklist_paths {
-	"/devices/platform/thinkpad_acpi/dytc_lapmode",
+	/* Some Lenovo machines have in-firmware thermal management,
+	 * avoid having two entities trying to manage things.
+	 * We may want to change this to dytc_perfmode once that is
+	 * widely available. */
+	"/sys/devices/platform/thinkpad_acpi/dytc_lapmode",
 };
 #endif
 
