@@ -727,8 +727,20 @@ int cthd_engine_default::read_cooling_devices() {
 			++current_cdev_index;
 
 			// Prefer MMIO access over MSR access for B0D4
-			if (rapl_dev)
+			if (rapl_dev) {
+				struct adaptive_target target = {};
+
 				rapl_dev->set_cdev_alias("");
+
+				if (adaptive_mode) {
+					thd_log_info("Disable rapl-msr interface and use rapl-mmio\n");
+					rapl_dev->rapl_update_enable_status(0);
+
+					target.code = "PL1MAX";
+					target.argument = "200000";
+					rapl_dev->set_adaptive_target(target);
+				}
+			}
 			rapl_mmio_dev->set_cdev_alias("B0D4");
 		} else {
 			delete rapl_mmio_dev;
@@ -818,6 +830,9 @@ int thd_engine_create_default_engine(bool ignore_cpuid_check,
 		bool exclusive_control, const char *conf_file) {
 	int res;
 	thd_engine = new cthd_engine_default();
+	if (!thd_engine)
+		return THD_ERROR;
+
 	if (exclusive_control)
 		thd_engine->set_control_mode(EXCLUSIVE);
 
@@ -826,7 +841,15 @@ int thd_engine_create_default_engine(bool ignore_cpuid_check,
 	if (conf_file)
 		thd_engine->set_config_file(conf_file);
 
-	res = thd_engine->thd_engine_start(ignore_cpuid_check);
+	res = thd_engine->thd_engine_init(ignore_cpuid_check);
+	if (res != THD_SUCCESS) {
+		if (res == THD_FATAL_ERROR)
+			thd_log_error("THD engine init failed\n");
+		else
+			thd_log_msg("THD engine init failed\n");
+	}
+
+	res = thd_engine->thd_engine_start();
 	if (res != THD_SUCCESS) {
 		if (res == THD_FATAL_ERROR)
 			thd_log_error("THD engine start failed\n");
@@ -857,6 +880,27 @@ void cthd_engine_default::workarounds()
 
 void cthd_engine_default::workaround_rapl_mmio_power(void)
 {
+	if (!workaround_enabled)
+		return;
+
+	cthd_cdev *cdev = search_cdev("rapl_controller_mmio");
+	if (cdev) {
+		/* RAPL MMIO is enabled and getting used. No need to disable */
+		return;
+	} else {
+		csys_fs _sysfs("/sys/devices/virtual/powercap/intel-rapl-mmio/intel-rapl-mmio:0/");
+
+		if (_sysfs.exists()) {
+			std::stringstream temp_str;
+
+			temp_str << "enabled";
+			if (_sysfs.write(temp_str.str(), 0) > 0)
+				return;
+
+			thd_log_debug("Failed to write to RAPL MMIO\n");
+		}
+	}
+
 #ifndef ANDROID
 	int map_fd;
 	void *rapl_mem;
@@ -868,9 +912,6 @@ void cthd_engine_default::workaround_rapl_mmio_power(void)
 
 	csys_fs sys_fs;
 
-	if (!workaround_enabled)
-		return;
-
 	ecx = edx = 0;
 	__cpuid(1, fms, ebx, ecx, edx);
 	family = (fms >> 8) & 0xf;
